#pragma version(1)
#pragma rs java_package_name(com.cvisionlab.testrenderscript.scripts)

#include "rs_utils.rsh"

static uchar y_lut[256];

static void calc_histogram(rs_allocation gray_img, int width, int height, unsigned int *histogram) {
    for (int y = 0; y < height; y++) {
        const uchar *gp = getRowUChar(gray_img, y);
        for (int x = 0; x < width; x++, gp++)
            histogram[*gp]++;
    }
}

static int get_first_significant_index(unsigned int *histogram, long s)
{
    for (int i = 0; i < 256; i++) {
        s -= histogram[i];
        if (s < 0)
            return i;
    }
    return -1;
}

static int get_last_significant_index(unsigned int *histogram, long s)
{
    for (int i = 255; i >= 0; i--) {
        s -= histogram[i];
        if (s < 0)
            return i;
    }
    return -1;
}

void set_params_custom_enhance(rs_allocation gray640_alloc, float hist_s) {

	const int W640 = rsAllocationGetDimX(gray640_alloc);
    const int H640 = rsAllocationGetDimY(gray640_alloc);

    unsigned int histogram[256];
    for (int i = 0; i < 256; i++)
        histogram[i] = 0;

    calc_histogram(gray640_alloc, W640, H640, histogram);

    hist_s = max(0.f, min(1.f, hist_s));
    const long histOffset = W640 * H640 * (hist_s / 2.f);

    const int min_ = get_first_significant_index(histogram, histOffset);
    const int max_ = get_last_significant_index(histogram, histOffset);

    unsigned int sum = 0;
    unsigned int prod_sum = 0;
    for (int i = min_; i <= max_; i++) {
        sum += histogram[i];
        prod_sum += i * histogram[i];
    }
    float mean_ = (float)prod_sum / sum;

    const int max_min = max(max_ - min_, 1);

    mean_ = max(128.f - 32.f, mean_);
    mean_ = min(128.f + 32.f, mean_);
    float power = log(0.5f) / log(mean_ / 255.f);

    for (int i = 0; i < min_; i++)
    	y_lut[i] = 0;

    for (int i = min_; i <= max_; i++)
    	y_lut[i] = pow((float)(i - min_) / max_min, power) * 255.f + 0.5f;

    for (int i = max_ + 1; i < 256; i++)
    	y_lut[i] = 255;
}

void root(const uchar4 *v_in, uchar4 *v_out, uint32_t x, uint32_t y) {
    v_out->r = y_lut[ v_in->r ];
    v_out->g = y_lut[ v_in->g ];
    v_out->b = y_lut[ v_in->b ];
    v_out->a = v_in->a;
/*
    *v_out = (uchar4){  red_lut[ master_lut[ SATURATE_CAST_TO_UCHAR(r1) ] ]  * noise / 255,
                        master_lut[ SATURATE_CAST_TO_UCHAR(g1) ]             * noise / 255,
                        blue_lut[ master_lut[ SATURATE_CAST_TO_UCHAR(b1) ] ] * noise / 255,
                        255  };*/

}