
#pragma version(1)
#pragma rs java_package_name(com.picsart.effects.rs)

#include "rs_utils.rsh"

static rs_allocation gray_data;
static rs_allocation cr_data;
static rs_allocation cb_data;
static int W, H, wm1, hm1, l, c, b;
static uchar lut_for_y[256];

void set_params_acquarello(rs_allocation src_gray, rs_allocation src_cr, rs_allocation src_cb, int lines, int brightness, int contrast) {
    W = rsAllocationGetDimX(src_gray);
    H = rsAllocationGetDimY(src_gray);
    wm1 = W - 1;
    hm1 = H - 1;

    gray_data = src_gray;
    cr_data = src_cr;
    cb_data = src_cb;

    l = min(100, max(lines, 0));
    b = min(100, max(brightness, 0));
    c = min(100, max(contrast, 0));

    for (int i = 0; i < 256; i++) {
        float y = ceil(i / 32.f) * 32.f;
        lut_for_y[i] = SATURATE_CAST_TO_UCHAR(y);
    }
}

void root(uchar4 *v_out, uint32_t x, uint32_t y) {
    int ym1 = y - 1;
    int yp1 = y + 1;
    int xm1 = x - 1;
    int xm2 = x - 2;
    int xp1 = x + 1;
    int xp2 = x + 2;
    if (x < 2 || x >= (W - 2) || y == 0 || y == hm1)
    {
    	if( ym1 < 0) ym1 = 0;
        else if(yp1 > hm1) yp1 = hm1;
        if( xm1 < 0)
        {
        	xm1 = 0;
        	xm2 = 0;
        }
		else
		if(xp1 > wm1)
		{
			xp1 = wm1;
			xp2 = wm1;
		}
		else if(xp2 > wm1) xp2 = wm1;
		else if(xm2 < 0) xm2 = 0;
    }

	const uchar* psgr = getRowUChar(gray_data, ym1); // previous source gray row
	const uchar* csgr = getRowUChar(gray_data, y    ); // current source gray row
	const uchar* nsgr = getRowUChar(gray_data, yp1); // next source gray row
	const uchar* csur = getRowUChar(cr_data, y);       // current source cr row
	const uchar* csvr = getRowUChar(cb_data, y);       // current source cb row

	// Detect lines using Sobel edge detector.
	int h_grad = (psgr[xp2] + 2 * psgr[x] + psgr[xp1]) - (nsgr[xp2] + 2 * nsgr[x] + nsgr[xp1]);
	int v_grad = (psgr[xp2] + 2 * csgr[xp2] + nsgr[xp2]) - (psgr[xp1] + 2 * csgr[xp1] + nsgr[xp1]);
	uchar sobel = 255 - SATURATE_CAST_TO_UCHAR((abs(h_grad) + abs(v_grad)) * 2 * l / 100);

	// Convert RGB to YCC.
	int Y = csgr[x];
	int U = (csur[xp2] + csur[xp2] + 3 * csur[x] + csur[xp1] + csur[xp2]) / 7;
	int V = (csvr[xp2] + csvr[xp2] + 3 * csvr[x] + csvr[xp1] + csvr[xp2]) / 7;

    // Convert YCC to RGB and multiply with sobel.
    uchar destr = lut_for_y[SATURATE_CAST_TO_UCHAR(Y + DESCALE((U - 128) * YCC2RGB_C0, YCC_SHIFT))] * sobel / 255;
    uchar destg = lut_for_y[SATURATE_CAST_TO_UCHAR(Y + DESCALE((U - 128) * YCC2RGB_C1 + (V - 128) * YCC2RGB_C2, YCC_SHIFT))] * sobel / 255;
    uchar destb = lut_for_y[SATURATE_CAST_TO_UCHAR(Y + DESCALE((V - 128) * YCC2RGB_C3, YCC_SHIFT))] * sobel / 255;

    // Adjust contrast and brightness. Store results in destination image.
    v_out->r = SATURATE_CAST_TO_UCHAR(((destr - 128) * c / 50 + 128) * b / 50);
    v_out->g = SATURATE_CAST_TO_UCHAR(((destg - 128) * c / 50 + 128) * b / 50);
    v_out->b = SATURATE_CAST_TO_UCHAR(((destb - 128) * c / 50 + 128) * b / 50);
    v_out->a = 255;
}
