
#pragma version(1)
#pragma rs java_package_name(com.picsart.effects.rs)

#include "rs_utils.rsh"

static rs_allocation gray_data;
static rs_allocation overlay_data;
static int W, H, wm1, hm1;
static uchar lut_for_y[256];
static uchar lut_for_lines[256];

void set_params_comicboom(rs_allocation src_gray, rs_allocation overlay, int lines, int brightness) {
    W = rsAllocationGetDimX(src_gray);
    H = rsAllocationGetDimY(src_gray);
    wm1 = W - 1;
    hm1 = H - 1;

    gray_data = src_gray;
    overlay_data = overlay;

    // Prepare lookup table to adjust lines.
    const int l = min(99, max(lines, 1));
    const float lk = 100.f / (100.f - l);
    const float lb = 255.f * l / (l - 100.f);
    for (int i = 0; i < 256; i++)
        lut_for_lines[i] = SATURATE_CAST_TO_UCHAR(lk * i + lb);

    // Prepare lookup table to adjust brightness.
    const int b = min(99, max(brightness, 1));
    float bb = 0.f;
    float bk = 0.f;
    if (b >= 50) {
        bb = 128.f * (b - 50.f) / 50.f;
        bk = (255.f - bb) / 255.f;
    }
    else {
        bk = (255.f + 128.f * (b - 50.f) / 50.f) / 255.f;
    }
    for (int i = 0; i < 256; i++) {
        float y = floor((i - 4.f) / 48.f + 0.5f) * 48.f + 4.f;    // posterize
        y = bk * y + bb;                                          // brightness
        lut_for_y[i] = SATURATE_CAST_TO_UCHAR(y);                 // store value to lookup table
    }
}

void root(const uchar4 *v_in, uchar4 *v_out, uint32_t x, uint32_t y) {
	int ym1 = y - 1;
    int yp1 = y + 1;
    int xm1 = x - 1;
    int xp1 = x + 1;
    if (x == 0 || x == wm1 || y == 0 || y == hm1) {
        if( ym1 < 0) ym1 = 0;
		else if(yp1 > hm1) yp1 = hm1;
		if( xm1 < 0) xm1 = 0;
        else if(xp1 > wm1) xp1 = wm1;
    }

    const uchar* psgr = getRowUChar(gray_data, ym1); // previous source gray row
	const uchar* csgr = getRowUChar(gray_data, y    ); // current source gray row
	const uchar* nsgr = getRowUChar(gray_data, yp1); // next source gray row
    const uchar4* cocp = getUChar4(overlay_data, y, x);  // current overlay color

    // Detect lines using Sobel edge detector.
    int h_grad = (psgr[xm1] + 2 * psgr[x] + psgr[xp1]) - (nsgr[xm1] + 2 * nsgr[x] + nsgr[xp1]);
    int v_grad = (psgr[xm1] + 2 * csgr[xm1] + nsgr[xm1]) - (psgr[xp1] + 2 * csgr[xp1] + nsgr[xp1]);
    uchar sobel = lut_for_lines[255 - SATURATE_CAST_TO_UCHAR(DIST(h_grad, v_grad) / 5.66f)];

    // Convert RGB to YCC.
    int Y = csgr[x];
    int U = DESCALE((v_in->r - Y) * RGB2YCC_C3 + YCC_DELTA, YCC_SHIFT);
    int V = DESCALE((v_in->b - Y) * RGB2YCC_C4 + YCC_DELTA, YCC_SHIFT);

    // Posterize Y and adjust brightness using lookup table.
    Y = lut_for_y[csgr[x]];

    // Convert YCC to RGB and multiply with sobel.
    v_out->r = cocp->r * SATURATE_CAST_TO_UCHAR(Y + DESCALE((U - 128) * YCC2RGB_C0, YCC_SHIFT)) * sobel / (255 * 255);
    v_out->g = cocp->g * SATURATE_CAST_TO_UCHAR(Y + DESCALE((U - 128) * YCC2RGB_C1 + (V - 128) * YCC2RGB_C2, YCC_SHIFT)) * sobel / (255 * 255);
    v_out->b = cocp->b * SATURATE_CAST_TO_UCHAR(Y + DESCALE((V - 128) * YCC2RGB_C3, YCC_SHIFT)) * sobel / (255 * 255);
    v_out->a = 255;
}
