
#pragma version(1)
#pragma rs java_package_name(com.picsart.effects.rs)

#include "rs_utils.rsh"

static int cx, cy;
static float radius;

static const uchar effect_40_RGB[] = {0, 0, 1, 1, 2, 3, 4, 4, 5, 5, 6, 6, 7, 8, 9, 10, 10, 11, 12, 13, 13, 14, 15, 16, 16, 17, 18, 18, 18, 19, 20, 21, 22, 22, 23, 24, 25, 25, 26, 27, 28, 28, 29, 30, 31, 32, 33, 34, 36, 37, 38, 39, 40, 41, 42, 43, 45, 46, 47, 48, 49, 50, 51, 52, 53, 55, 56, 57, 58, 59, 60, 61, 62, 63, 65, 66, 67, 68, 69, 70, 71, 72, 74, 75, 76, 77, 78, 79, 80, 81, 82, 84, 85, 86, 87, 88, 90, 91, 92, 93, 95, 96, 97, 98, 99, 100, 101, 102, 104, 105, 106, 107, 108, 109, 110, 111, 112, 114, 115, 117, 118, 119, 120, 121, 122, 124, 125, 126, 127, 128, 129, 130, 131, 132, 134, 135, 136, 138, 139, 140, 141, 142, 143, 145, 146, 147, 148, 149, 150, 151, 152, 154, 155, 156, 157, 158, 159, 160, 161, 162, 165, 166, 167, 168, 169, 170, 171, 172, 174, 175, 176, 177, 178, 179, 180, 181, 182, 184, 185, 186, 187, 188, 189, 190, 191, 192, 194, 195, 196, 197, 198, 199, 200, 201, 203, 204, 205, 206, 207, 208, 209, 210, 211, 213, 214, 215, 216, 217, 218, 219, 220, 221, 223, 224, 225, 226, 227, 228, 229, 230, 232, 233, 234, 235, 235, 236, 236, 237, 238, 238, 239, 240, 240, 241, 241, 242, 243, 243, 244, 245, 245, 246, 247, 247, 248, 248, 248, 249, 249, 250, 251, 251, 252, 253, 253, 254};

static void gradient_map(uchar4 * dp) {
    const int r0 = 239;
    const int g0 = 193;
    const int b0 = 52;

    const int dr = 16;
    const int dg = -69;
    const int db = -52;

    int r = dp->r;
    int g = dp->g;
    int b = dp->b;

    int Y = PST_GRADIENT_SMOOTH_100_TABLE[ DESCALE(r * RGB2YCC_C0  +  g * RGB2YCC_C1  +  b * RGB2YCC_C2, YCC_SHIFT) ];

    // Blend with soft light
    int tr = r - (255 - 2 * (r0 + dr * Y / 255)) * r * (255 - r) / 65025;
    int tg = g - (255 - 2 * (g0 + dg * Y / 255)) * g * (255 - g) / 65025;
    int tb = b - (255 - 2 * (b0 + db * Y / 255)) * b * (255 - b) / 65025;

    // Opacity 40%
    dp->r = (tr * 4 + r * 6) / 10;
    dp->g = (tg * 4 + g * 6) / 10;
    dp->b = (tb * 4 + b * 6) / 10;
}

static void radial_gradient_blend(uchar4* p, const int alpha, const int oma, const int gr, const int gg, const int gb) {
    const int r = p->r;
    const int g = p->g;
    const int b = p->b;

    int tr = r + gr - (r * gr) / 255;
    int tg = g + gg - (g * gg) / 255;
    int tb = b + gb - (b * gb) / 255;

    tr = (tr * oma + r * alpha) / 255;
    tg = (tg * oma + g * alpha) / 255;
    tb = (tb * oma + b * alpha) / 255;

    p->r = (tr + r) / 2;
    p->g = (tg + g) / 2;
    p->b = (tb + b) / 2;
}

static void effect_40_radial_gradient(uchar4* p, uint32_t x, uint32_t y) {
    const int r0 = 255;
    const int g0 = 72;
    const int b0 = 0;

    const int dr = 255 - r0;
    const int dg = 150 - g0;
    const int db = 0 - b0;

    const int dx = cx - x;
    const int dy = cy - y;

    const int k = PST_GRADIENT_SMOOTH_100_TABLE[ min((int)(sqrt((float)(dx*dx + dy*dy)) / radius * 255.f + 0.5f), 255) ];
    const int gr = r0 + dr * k / 255;
    const int gg = g0 + dg * k / 255;
    const int gb = b0 + db * k / 255;
    const int oma = 255 - k;

    radial_gradient_blend(p, k, oma, gr, gg, gb);
}

void set_params_effect_40(int w, int h) {
    const float angle = 0.f;
    const float scale = 1.5f;
    const float offset_x = 0.f;
    const float offset_y = 0.f;

    const int image_center_x = (w + 1) / 2;
    const int image_center_y = (h + 1) / 2;

    cx = image_center_x + offset_x * w;
    cy = image_center_y + offset_y * h;

    // Intersection point is point in which angled beam from center of image intersects edge of image.
    float intersection_x;
    float intersection_y;
    const float corner_angle = atan2((float)h, (float)w);

    if (angle < corner_angle) {
        intersection_x = w;
        intersection_y = image_center_y - tan(angle) * image_center_x;
    }
    else {
        intersection_x = image_center_x + tan(M_PI_2 - angle) * image_center_y;
        intersection_y = 0;
    }

    // Inverted & scaled distance between points.
    const float distance = sqrt(pow(image_center_x - intersection_x, 2.f) + pow(image_center_y - intersection_y, 2.f));
    radius =  distance * scale;
}

void root(uchar4 *v_out, uint32_t x, uint32_t y) {
    uchar4 val = *v_out;

    // Curves, Brightness/Contrast
    {
        val.r = effect_40_RGB[val.r];
        val.g = effect_40_RGB[val.g];
        val.b = effect_40_RGB[val.b];
    }

    // Gradient Map
    {
        gradient_map(&val);
    }

    // Gradient fill layer, color1:#FF4800 color2: #FF9600; opacity:50% blending: screen.
    {
        effect_40_radial_gradient(&val, x, y);
    }

    *v_out = val;
}
