
#pragma version(1)
#pragma rs java_package_name(com.picsart.effects.rs)

#include "rs_utils.rsh"

static rs_allocation gray_data;
static rs_allocation overlay1_gray;
static rs_allocation overlay2_gray;
static int W, H, wm1, hm1, l, c, b;

void set_params_grannyspaper(rs_allocation src_gray, rs_allocation gray_overlay1, rs_allocation gray_overlay2, int lines, int brightness, int contrast) {
    W = rsAllocationGetDimX(src_gray);
    H = rsAllocationGetDimY(src_gray);
    wm1 = W - 1;
    hm1 = H - 1;

    gray_data = src_gray;
    overlay1_gray = gray_overlay1;
    overlay2_gray = gray_overlay2;

    l = min(100, max(lines, 0));
    b = min(100, max(brightness, 0));
    c = min(100, max(contrast, 0));
}

void root(uchar4 *v_out, uint32_t x, uint32_t y) {
	int ym1 = y - 1;
    int yp1 = y + 1;
    int xm1 = x - 1;
    int xp1 = x + 1;
    if (x == 0 || x == wm1 || y == 0 || y == hm1) {
        if( ym1 < 0) ym1 = 0;
		else if(yp1 > hm1) yp1 = hm1;
		if( xm1 < 0) xm1 = 0;
        else if(xp1 > wm1) xp1 = wm1;
    }

    const uchar* psgr = getRowUChar(gray_data, ym1); // previous source gray row
	const uchar* csgr = getRowUChar(gray_data, y    ); // current source gray row
	const uchar* nsgr = getRowUChar(gray_data, yp1); // next source gray row

    // Detect lines using Sobel edge detector.
    int h_grad = (psgr[xm1] + 2 * psgr[x] + psgr[xp1]) - (nsgr[xm1] + 2 * nsgr[x] + nsgr[xp1]);
    int v_grad = (psgr[xm1] + 2 * csgr[xm1] + nsgr[xm1]) - (psgr[xp1] + 2 * csgr[xp1] + nsgr[xp1]);
    uchar edge = 255 - SATURATE_CAST_TO_UCHAR((abs(h_grad) + abs(v_grad)) * l / 50);

    // Blend edges with with overlay.
    const int Y = csgr[x];
    if (Y < 26)
        edge = edge * getValUChar(overlay1_gray, (y + H) / 2,  (x + W) / 2) / 255;
    else if (Y < 77)
        edge = edge * getValUChar(overlay1_gray, (y + H) / 2,  (x    ) / 2) / 255;
    else if (Y < 128)
        edge = edge * getValUChar(overlay1_gray, (y    ) / 2,  (x + W) / 2) / 255;
    else if (Y < 179)
        edge = edge * getValUChar(overlay1_gray, (y    ) / 2,  (x    ) / 2) / 255;

    // Colorify, adjust contrast and brightness.
    uchar o2 = getValUChar(overlay2_gray, y, x);
    uchar destr = SATURATE_CAST_TO_UCHAR(((edge * o2 * 193 / 65025 - 128) * c / 50 + 128) * b / 50);
    uchar destg = SATURATE_CAST_TO_UCHAR(((edge * o2 * 154 / 65025 - 128) * c / 50 + 128) * b / 50);
    uchar destb = SATURATE_CAST_TO_UCHAR(((edge * o2 * 107 / 65025 - 128) * c / 50 + 128) * b / 50);

    float dx = (float)x / W - 0.5f;
    float dy = (float)y / H - 0.5f;
    float coef = 0.9f - sqrt(dx * dx + dy * dy);

    // Store results in destination image.
    v_out->r = destr * coef;
    v_out->g = destg * coef;
    v_out->b = destb * coef;
    v_out->a = 255;
}
