#pragma version(1)
#pragma rs java_package_name(com.picsart.effects.rs)

#include "rs_utils.rsh"

typedef struct Box
{
	int2 rb;
	int2 rt;
	int2 lb;
	int2 lt;
} Box;

typedef struct Shape
{
	int R;
	int h;
	int v;
	int d;
	int size;
} Shape;

Box * box_array;
static int box_count;
static Shape data;
static rs_allocation integral;
static int W,H;

static void create_square_d()
{
	int d = data.d;
	int R = data.R;
	int d2 = (d-1)/2;
	int b = R/d;
	R = data.R = d*b + d2;

	data.v = data.h = R;
	data.size = 0;

	for (int i = 0; i<b; ++i)
	{
		Box boxl;
		Box boxr;

		boxl.rb.x = boxl.rt.x = -R - 1 + d*(i+1);
		boxl.lb.x = boxl.lt.x = -R - 1 + i*d;
		boxl.rb.y = boxl.lb.y = d2 + i*d;
		boxl.rt.y = boxl.lt.y = -d2 - 1 - i*d;

		boxr.rb.x = boxr.rt.x = R - i*d;
		boxr.lb.x = boxr.lt.x = R - d*(1 + i);
		boxr.rb.y = boxr.lb.y = d2 + i*d;
		boxr.rt.y = boxr.lt.y = -d2 - 1 - i*d;

		box_array[i] = boxl;
		box_array[box_count-i-1] = boxr;

		data.size += 2*(boxl.rb.x - boxl.lb.x)*(boxl.rb.y-boxl.rt.y);
	}

	Box boxc;

	boxc.rb.x = boxc.rt.x = d2;
	boxc.rb.y = boxc.lb.y = R;
	boxc.lb.x = boxc.lt.x = -d2 - 1;
	boxc.rt.y = boxc.lt.y = -R - 1;

	data.size += (boxc.rb.x - boxc.lb.x)*(boxc.rb.y-boxc.rt.y);

	box_array[b] = boxc;
}

static void create_octagon_d()
{
	int d = data.d;
	int b = (int) (data.R * sqrt(sqrt(2.f) - 1.f));
	b = b/d;

	int a = (int) (sqrt(2.f) * b*d);
	a = (a/2)*2+1;
	int a2 = a/2 + d/2 + d%2;
	int h = data.h = b*d+a2;
	int v = data.v = data.h;

	data.size = 0;

	for (int i = 0; i<b; ++i)
	{
		Box boxl;
		Box boxr;

		boxl.rb.x = boxl.rt.x = -h - 1 + (i+1)*d;
		boxl.lb.x = boxl.lt.x = -h - 1 +i*d;

		boxr.rb.x = boxr.rt.x = h - i*d;
		boxr.lb.x = boxr.lt.x = h - (i+1)*d;

		boxl.rb.y = boxl.lb.y = boxr.rb.y = boxr.lb.y = a2 + i*d;
		boxl.rt.y = boxl.lt.y = boxr.rt.y = boxr.lt.y = -a2 - 1 - i*d;

		box_array[i] = boxl;
		box_array[box_count-i-1] = boxr;

		data.size += 2*(boxl.rb.x - boxl.lb.x)*(boxl.rb.y-boxl.rt.y);
	}

	Box boxc;
	boxc.rb.x = boxc.rt.x = a2;
	boxc.rb.y = boxc.lb.y = v;
	boxc.lb.x = boxc.lt.x = -a2 - 1;
	boxc.rt.y = boxc.lt.y = -v - 1;
	box_array[b] = boxc;

	data.size += (boxc.rb.x - boxc.lb.x)*(boxc.rb.y-boxc.rt.y);
}

static void create_triangle_d()
{
	int d = data.d;
	int R = data.R;
	int d2 = (d-1)/2;

	float sqrt3 = sqrt(3.f);

	int a2 = (int) (sqrt3*R/2.0);
	int b = a2/d;
	a2 = data.h = d*b + d2;

	int v = (sqrt3 * a2)/2;
	data.v = v;

	data.size = 0;


	for (int i = 0; i<b; ++i)
	{
		Box boxl;
		Box boxr;

		boxl.rb.x = boxl.rt.x = -a2 - 1 + d*(i+1);
		boxl.lb.x = boxl.lt.x = -a2 - 1 + d*i;
		boxr.rb.x = boxr.rt.x = a2 - d*i;
		boxr.lb.x = boxr.lt.x = a2 - d*(i+1);

		boxl.rb.y = boxl.lb.y = boxr.rb.y = boxr.lb.y = v;
		boxl.rt.y = boxl.lt.y = boxr.rt.y = boxr.lt.y = boxl.lb.y - (int) (sqrt3*(d*i+d/2));

		box_array[i] = boxl;
		box_array[box_count-i-1] = boxr;

		data.size += 2*(boxl.rb.x - boxl.lb.x)*(boxl.rb.y-boxl.rt.y);
	}

	Box boxc;
	boxc.rb.x = boxc.rt.x = d2;
	boxc.rb.y = boxc.lb.y = v;
	boxc.lb.x = boxc.lt.x = -d2 - 1;
	boxc.rt.y = boxc.lt.y = -v - 1;
	box_array[b] = boxc;

	data.size += (boxc.rb.x - boxc.lb.x)*(boxc.rb.y-boxc.rt.y);
}

void create_shape(rs_allocation integ, int R, int delta, int shape, int box_c)
{
	integral = integ;
	box_count = box_c;

	W = rsAllocationGetDimX(integral);
	H = rsAllocationGetDimY(integral);

	data.R = R;
	data.d = delta;

	switch( shape )
	{
	case 0:
		create_octagon_d();
		break;
	case 1:
		create_square_d();
		break;
	case 2:
		create_triangle_d();
		break;
	}
}

void root(int3 *v_out, uint32_t x, uint32_t y)
{

	const int di = y;
	const int dj = x;

	uint3 res = 0;
	int size = 0;

	if( y <= data.v || y >= H - data.v || x <= data.h || x >= W - data.h)
	{
		for (int i = 0; i<box_count; ++i)
		{
			Box box;

			box.rb.y = box_array[i].rb.y + di;
			box.lb.y = box_array[i].lb.y + di;
			box.rt.y = box_array[i].rt.y + di;
			box.lt.y = box_array[i].lt.y + di;
			box.rb.x = box_array[i].rb.x + dj;
			box.lb.x = box_array[i].lb.x + dj;
			box.rt.x = box_array[i].rt.x + dj;
			box.lt.x = box_array[i].lt.x + dj;

			if(box.rb.x >= 0 && box.lb.x < W-1 && box.rt.y < H-1 && box.rb.y >= 0)
			{
				if(box.lb.y >= H)
				{
					box.lb.y = box.rb.y = H-1;
				}

				if(box.rb.x >= W)
				{
					box.rb.x = box.rt.x = W-1;
				}

				res += rsGetElementAt_uint3(integral, box.rb.x, box.rb.y);

				size += (box.rb.x + 1) * (box.rb.y + 1);

				if(box.lb.x >= 0)
				{
					res -= rsGetElementAt_uint3(integral, box.lb.x, box.lb.y);
					size -= (box.lb.x + 1) * (box.lb.y + 1);

					if(box.rt.y >= 0)
					{
						res += rsGetElementAt_uint3(integral, box.lt.x, box.lt.y);
						size += (box.lt.x + 1) * (box.lt.y + 1);
					}
				}
				if(box.rt.y >= 0)
				{
					res -= rsGetElementAt_uint3(integral, box.rt.x, box.rt.y);
					size -= (box.rt.x + 1) * (box.rt.y + 1);
				}
			}
		}
	}
	else
	{
		for (int i = 0; i < box_count; ++i)
		{
			Box box;

			box.rb.y = box_array[i].rb.y + di;
			box.lb.y = box_array[i].lb.y + di;
			box.rt.y = box_array[i].rt.y + di;
			box.lt.y = box_array[i].lt.y + di;
			box.rb.x = box_array[i].rb.x + dj;
			box.lb.x = box_array[i].lb.x + dj;
			box.rt.x = box_array[i].rt.x + dj;
			box.lt.x = box_array[i].lt.x + dj;

			res += rsGetElementAt_uint3(integral, box.rb.x, box.rb.y) - rsGetElementAt_uint3(integral, box.lb.x, box.lb.y)
					- rsGetElementAt_uint3(integral, box.rt.x, box.rt.y) + rsGetElementAt_uint3(integral, box.lt.x, box.lt.y);

		}

		size = data.size;
	}

	v_out->s0 = res.r/size;
	v_out->s1 = res.g/size;
	v_out->s2 = res.b/size;
}

