
#pragma version(1)
#pragma rs java_package_name(com.picsart.effects.rs)

#include "rs_utils.rsh"

static uchar red_lut[256];
static uchar green_lut[256];
static uchar blue_lut[256];
static int x0, y0;
static float ss2;

void set_params_lomo(int w, int h, float vignette, float contrast, float amount, int mode) {
    vignette = max(0.f, min(1.f, vignette));
    contrast = max(0.f, min(1.f, contrast));
    amount   = max(0.f, min(1.f, amount));

    // Prepare LUT for color correction and contrast adjustment.
    int mode_r = mode / 100;
    int mode_g = mode / 10 - 10 * (mode / 100);
    int mode_b = mode - 10 * (mode / 10);

    Corrector cor_r = build_corrector(mode_r, amount);
    Corrector cor_g = build_corrector(mode_g, amount);
    Corrector cor_b = build_corrector(mode_b, amount);

    const int b = 255.f * (1.f - contrast / 2.f);
    const int a = 255 - b;

    const uchar cor_r_0 = cor_r.correct(0, cor_r.param);
    const uchar cor_g_0 = cor_g.correct(0, cor_g.param);
    const uchar cor_b_0 = cor_b.correct(0, cor_b.param);

    for (int i = 0; i < a; i++) {
        red_lut[i] = cor_r_0;
        green_lut[i] = cor_g_0;
        blue_lut[i] = cor_b_0;
    }

    for (int i = a; i <= b; i++) {
        red_lut[i] = cor_r.correct(255.f * (i - a) / (b - a), cor_r.param);;
        green_lut[i] = cor_g.correct(255.f * (i - a) / (b - a), cor_g.param);
        blue_lut[i] = cor_b.correct(255.f * (i - a) / (b - a), cor_b.param);
    }

    const uchar cor_r_255 = cor_r.correct(255, cor_r.param);
    const uchar cor_g_255 = cor_g.correct(255, cor_g.param);
    const uchar cor_b_255 = cor_b.correct(255, cor_b.param);

    for (int i=(b+1); i<256; i++) {
        red_lut[i] = cor_r_255;
        green_lut[i] = cor_g_255;
        blue_lut[i] = cor_b_255;
    }

    const float s = 2 * max(w, h) * vignette;
    ss2 = s*s*2;
    x0 = w / 2;
    y0 = h / 2;
}

void root(const uchar4 *v_in, uchar4 *v_out, uint32_t x, uint32_t y) {
    const int dx = x - x0;
    const int dy = y - y0;
    const float arg = -(dx*dx + dy*dy)/ss2;
    const float c = exp(arg);

    *v_out = (uchar4){c * red_lut[v_in->r], c * green_lut[v_in->g], c * blue_lut[v_in->b], 255};
}

