
#pragma version(1)
#pragma rs java_package_name(com.picsart.effects.rs)

#include "rs_utils.rsh"

static rs_allocation gray_data;
static int W, H, l, c, b;
static uchar lut_for_y[256];

void set_params_neoncola(rs_allocation src_gray, int lines, int brightness, int contrast) {
    W = rsAllocationGetDimX(src_gray);
    H = rsAllocationGetDimY(src_gray);

    gray_data = src_gray;

    l = min(100, max(lines, 0));
    b = min(100, max(brightness, 0));
    c = min(100, max(contrast, 0));

    for (int i = 0; i < 256; i++) {
        float y = (ceil((i * 256.f / 64.f) - 0.5f) * 64.f) / 256.f;
        lut_for_y[i] = SATURATE_CAST_TO_UCHAR(y);
    }
}

void root(const uchar4 *v_in, uchar4 *v_out, uint32_t x, uint32_t y) {
    if (x == 0 || x == (W - 1) || y == 0 || y == (H - 1)) {
        *v_out = (uchar4){0, 0, 0, 255};
        return;
    }

    const uchar* psgr = getRowUChar(gray_data, y - 1); // previous source gray row
    const uchar* csgr = getRowUChar(gray_data, y    ); // current source gray row
    const uchar* nsgr = getRowUChar(gray_data, y + 1); // next source gray row

    // Detect lines using Sobel edge detector.
    int h_grad = (psgr[x - 1] + 2 * psgr[x] + psgr[x + 1]) - (nsgr[x - 1] + 2 * nsgr[x] + nsgr[x + 1]);
    int v_grad = (psgr[x - 1] + 2 * csgr[x - 1] + nsgr[x - 1]) - (psgr[x + 1] + 2 * csgr[x + 1] + nsgr[x + 1]);
    uchar sobel = SATURATE_CAST_TO_UCHAR((abs(h_grad) + abs(v_grad)) * 2 * l / 100);

    // Convert RGB to YCC.
    int Y = csgr[x];
    int U = DESCALE((v_in->r - Y) * RGB2YCC_C3 + YCC_DELTA, YCC_SHIFT);
    int V = DESCALE((v_in->b - Y) * RGB2YCC_C4 + YCC_DELTA, YCC_SHIFT);

    // Posterize Y.
    Y = lut_for_y[Y];

    // Convert YCC to RGB and adjust contrast.
    const int red   = (Y + DESCALE((U - 128) * YCC2RGB_C0, YCC_SHIFT) - 128)                          * c / 50 + 128;
    const int green = (Y + DESCALE((U - 128) * YCC2RGB_C1 + (V - 128) * YCC2RGB_C2, YCC_SHIFT) - 128) * c / 50 + 128;
    const int blue  = (Y + DESCALE((V - 128) * YCC2RGB_C3, YCC_SHIFT) - 128)                          * c / 50 + 128;

    // Multiply with sobel. Adjust brightness. Store results in destination image.
    v_out->r = SATURATE_CAST_TO_UCHAR(red   * sobel * b / 12750);
    v_out->g = SATURATE_CAST_TO_UCHAR(green * sobel * b / 12750);
    v_out->b = SATURATE_CAST_TO_UCHAR(blue  * sobel * b / 12750);
    v_out->a = 255;
}
