
#pragma version(1)
#pragma rs java_package_name(com.picsart.effects.rs)

#include "rs_utils.rsh"

static rs_allocation gray_data, overlay_gray;
static int W, H, wm1, hm1, l, c, b;
static uchar lut_for_y[256];

void set_params_pastelperfect(rs_allocation src_gray, rs_allocation gray_overlay, int lines, int brightness, int contrast) {
    W = rsAllocationGetDimX(src_gray);
    H = rsAllocationGetDimY(src_gray);
    wm1 = W - 1;
    hm1 = H - 1;

    gray_data = src_gray;
    overlay_gray = gray_overlay;

    l = min(100, max(lines, 0));
    b = min(100, max(brightness, 0));
    c = min(100, max(contrast, 0));

    for (int i = 0; i < 256; i++)
        lut_for_y[i] = SATURATE_CAST_TO_UCHAR(ceil((i / 48.f) - 0.5f) * 48.f);
}

void root(const uchar4 *v_in, uchar4 *v_out, uint32_t x, uint32_t y) {
	int ym1 = y - 1;
    int yp1 = y + 1;
    int xm1 = x - 1;
    int xp1 = x + 1;
    if (x == 0 || x == wm1 || y == 0 || y == hm1) {
        if( ym1 < 0) ym1 = 0;
		else if(yp1 > hm1) yp1 = hm1;
		if( xm1 < 0) xm1 = 0;
        else if(xp1 > wm1) xp1 = wm1;
    }

    const uchar* psgr = getRowUChar(gray_data, ym1); // previous source gray row
	const uchar* csgr = getRowUChar(gray_data, y    ); // current source gray row
	const uchar* nsgr = getRowUChar(gray_data, yp1); // next source gray row

    // Detect lines using Sobel edge detector.
    int h_grad = (psgr[xm1] + 2 * psgr[x] + psgr[xp1]) - (nsgr[xm1] + 2 * nsgr[x] + nsgr[xp1]);
    int v_grad = (psgr[xm1] + 2 * csgr[xm1] + nsgr[xm1]) - (psgr[xp1] + 2 * csgr[xp1] + nsgr[xp1]);
    int result = SATURATE_CAST_TO_UCHAR((abs(h_grad) + abs(v_grad)) * l / 50);

    // Adjust edges.
    result = 255 - result * (382 - result) / 255;

    // Convert RGB to YCC.
    int Y = csgr[x];
    int U = DESCALE((v_in->r - Y) * RGB2YCC_C3 + YCC_DELTA, YCC_SHIFT);
    int V = DESCALE((v_in->b - Y) * RGB2YCC_C4 + YCC_DELTA, YCC_SHIFT);

    // Posterize Y and Convert posterized YCC to RGB.
    uchar posty = lut_for_y[Y];
    uchar postr = SATURATE_CAST_TO_UCHAR(posty + DESCALE((U - 128) * YCC2RGB_C0, YCC_SHIFT));
    uchar postg = SATURATE_CAST_TO_UCHAR(posty + DESCALE((U - 128) * YCC2RGB_C1 + (V - 128) * YCC2RGB_C2, YCC_SHIFT));
    uchar postb = SATURATE_CAST_TO_UCHAR(posty + DESCALE((V - 128) * YCC2RGB_C3, YCC_SHIFT));

    // Blend edges with with overlay.
    if (Y < 26)
        result = result * getValUChar(overlay_gray, (y + H) / 2, (x + W) / 2) / 255;
    else if (Y < 77)
        result = result * getValUChar(overlay_gray, (y + H) / 2, (x    ) / 2) / 255;
    else if (Y < 128)
        result = result * getValUChar(overlay_gray, (y    ) / 2, (x + W) / 2) / 255;
    else
        result = result * getValUChar(overlay_gray, (y    ) / 2, (x    ) / 2) / 255;

    // Colorify and blend.
    uchar destr = (postr * 230 * 35 / 255 + result * 65) / 100;
    uchar destg = (postg * 160 * 35 / 255 + result * 65) / 100;
    uchar destb = (postb * 100 * 35 / 255 + result * 65) / 100;

    // Adjust contrast and brightness. Store results in destination image.
    v_out->r = SATURATE_CAST_TO_UCHAR(((destr - 128) * c / 50 + 128) * b / 50);
    v_out->g = SATURATE_CAST_TO_UCHAR(((destg - 128) * c / 50 + 128) * b / 50);
    v_out->b = SATURATE_CAST_TO_UCHAR(((destb - 128) * c / 50 + 128) * b / 50);
    v_out->a = 255;
}
