
#pragma version(1)
#pragma rs java_package_name(com.picsart.effects.rs)

#include "rs_utils.rsh"

static const int memal_max = 1024;

typedef struct COctTree_ {
    struct COctTree_	**_kids;		//!< child nodes
	int                 _weight;		//!< weight of current node
	int                 _kids_num;		//!< count of childs
	int                 _depth;			//!< current depth
	int                 _max_depth;		//!< maximum depth relative current node
	int                 _clr[3];		//!< color of current node
	int                 _validate;		//!< validation flag
	bool                _avr;			//!< average flag
} COctTree;

static COctTree octtree, memal[memal_max], *pmemal[memal_max];
static int memal_top;

static const int sw = 50;
static const int sh = 50;

static int COctTree_GetChild(const uchar4 clr, int depth) {
    int r = (clr.r >> (7 - depth)) & 1;
    int g = (clr.g >> (7 - depth)) & 1;
    int b = (clr.b >> (7 - depth)) & 1;

    return (r << 0) + (g << 1) + (b << 2);
}

static void COctTree_Average(COctTree *this, bool avr) {
    this->_avr = avr;
}

static void COctTree_Empty(COctTree *this) {
	if(this->_kids != NULL) {
		for(int i = 0; i < 8; ++i) {
            COctTree_Empty(this->_kids[i]);
        }
        this->_kids = NULL;
		this->_kids_num = 0;
	}
}

static bool COctTree_IsValidate(COctTree *this) {
    return this->_validate == 42;
}

static COctTree* COctTree_Create() {
    COctTree* this = pmemal[memal_top];
    ++memal_top;
    this->_weight = 0;
    this->_kids_num = 0;
    this->_depth = 0;
    this->_max_depth = 0;
    this->_clr[0] = 0;
    this->_clr[1] = 0;
    this->_clr[2] = 0;
    this->_validate = 0;
    this->_avr = false;
    this->_kids = NULL;
    return this;
}

static void  COctTree_Initialize(COctTree* this, int max_depth, int depth) {
    this->_weight = 0;
    this->_kids_num = 0;
    this->_depth = depth;
    this->_max_depth = max_depth;
    this->_clr[0] = 0;
    this->_clr[1] = 0;
    this->_clr[2] = 0;
    this->_validate = 42;
    this->_avr = false;
    this->_kids = NULL;
}

static void COctTree_Insert(COctTree *this, uchar4 clr) {

    if (this->_kids == NULL && this->_max_depth > 1 && this->_weight == 1) {
        this->_kids = pmemal + memal_top;
        for(int i = 0; i < 8; ++i)
            this->_kids[i] = COctTree_Create();

        const uchar4 _clr888 = {this->_clr[0], this->_clr[1], this->_clr[2], 255};

        int cur_index = COctTree_GetChild(_clr888, this->_depth);
        if (!COctTree_IsValidate(this->_kids[cur_index])) {
            COctTree_Initialize(this->_kids[cur_index], this->_max_depth - 1, this->_depth + 1);
        	this->_kids_num++;
        }
        COctTree_Insert(this->_kids[cur_index], _clr888);
    }
    if (this->_kids != NULL) {
        int cur_index = COctTree_GetChild(clr, this->_depth);
        if (!COctTree_IsValidate(this->_kids[cur_index])) {
            COctTree_Initialize(this->_kids[cur_index], this->_max_depth - 1, this->_depth + 1);
        	this->_kids_num++;
        }
        COctTree_Insert(this->_kids[cur_index], clr);
    }
    this->_clr[0] += clr.r;
    this->_clr[1] += clr.g;
    this->_clr[2] += clr.b;
    this->_weight++;
}

static void COctTree_Reduce(COctTree *this, int steps) {
    if (this->_kids == NULL)
        return;

    if (steps == 1) {
    	COctTree_Empty(this);
        return;
    }

    int childsCount = 0;
    for (int i = 0; i < 8; ++i)
        if (COctTree_IsValidate(this->_kids[i]))
            childsCount++;

    int childs_index[8] = {3, 1, 2, 5, 4, 6, 0, 7};

    this->_clr[0] = 0;
    this->_clr[1] = 0;
    this->_clr[2] = 0;
    this->_weight = 0;

    for (int i = 0; i < 8; ++i) {
        int cur_child = childs_index[i];
        if (COctTree_IsValidate(this->_kids[cur_child])) {
        	int dstep = steps / childsCount;
            if (dstep != 0) {
                COctTree_Reduce(this->_kids[cur_child], dstep);
                this->_clr[0] += this->_kids[cur_child]->_clr[0];
                this->_clr[1] += this->_kids[cur_child]->_clr[1];
                this->_clr[2] += this->_kids[cur_child]->_clr[2];
                this->_weight += this->_kids[cur_child]->_weight;
                steps = steps - dstep;
            }
            else {
            	this->_clr[0] += this->_kids[cur_child]->_clr[0];
            	this->_clr[1] += this->_kids[cur_child]->_clr[1];
            	this->_clr[2] += this->_kids[cur_child]->_clr[2];
                this->_weight += this->_kids[cur_child]->_weight;

                COctTree_Empty(this->_kids[cur_child]);
                this->_kids[cur_child]->_validate = 0;
                this->_kids_num--;
            }
            childsCount--;
        }
    }
}

static void COctTree_Map(const COctTree *this, const uchar4 * clr, uchar4 * dst) {
    if (this->_kids == NULL) {
        dst->r = SATURATE_CAST_TO_UCHAR(this->_clr[0] / this->_weight);
        dst->g = SATURATE_CAST_TO_UCHAR(this->_clr[1] / this->_weight);
        dst->b = SATURATE_CAST_TO_UCHAR(this->_clr[2] / this->_weight);
        dst->a = 255;
        return;
    }

    int cur_index = COctTree_GetChild(*clr, this->_depth);
    if (COctTree_IsValidate(this->_kids[cur_index])) {
        COctTree_Map(this->_kids[cur_index], clr, dst);
        return;
    }

    if (this->_avr) {
        int arr[8][7] = {	{1, 2, 4, 3, 5, 6, 7},
                            {2, 3, 5, 0, 7, 5, 4},
                            {3, 1, 6, 0, 7, 4, 5},
                            {2, 1, 5, 0, 7, 4, 6},
                            {1, 5, 6, 0, 7, 2, 3},
                            {4, 1, 3, 0, 7, 6, 2},
                            {2, 4, 3, 7, 0, 5, 1},
                            {6, 5, 3, 4, 2, 1, 0}};
        for (int i = 0; i < 7; ++i) {
            if (COctTree_IsValidate(this->_kids[arr[cur_index][i]])) {
                COctTree_Map(this->_kids[arr[cur_index][i]], clr, dst);
                return;
            }
        }
    }

    dst->r = SATURATE_CAST_TO_UCHAR(this->_clr[0] / this->_weight);
    dst->g = SATURATE_CAST_TO_UCHAR(this->_clr[1] / this->_weight);
    dst->b = SATURATE_CAST_TO_UCHAR(this->_clr[2] / this->_weight);
    dst->a = 255;
}


void set_params_posterize(rs_allocation small, int colors) {
    memal_top = 0;
    COctTree_Initialize(&octtree, 4, 0);

    for(int i = 0; i < memal_max; ++i)
    	pmemal[i] = &memal[i];

	for (int y = 0; y < sh; ++y) {
        uchar4 *sip = getRowUChar4(small, y);
		for (int x = 0; x < sw; ++x)
		{
			uchar4 color = *(sip + x);
            COctTree_Insert(&octtree, color);
		}
    }

    if (colors > 2)
    	COctTree_Reduce(&octtree, colors-1);
    else {
    	COctTree_Average(&octtree, true);
    	COctTree_Reduce(&octtree, 2);
    }
}

void root(uchar4 *v_out, uint32_t x, uint32_t y) {
	COctTree_Map(&octtree, v_out, v_out);
}
