#pragma version(1)
#pragma rs java_package_name(com.picsart.effects.rs)

#define SATURATE_CAST_TO_UCHAR(v) ((uchar)((v) <= 0 ? 0 : ((v) >= 255 ? 255 : (v))))

static rs_allocation diffs[8];

static int width;
static int height;

static int labWidth;
static int labHeight;
static float toLabScale;
static float labToBitmapScale;
static int adjustPointsCount;
static float maxTollerance;

int* centerXs;
int* centerYs;
int* radiuses;
int* minRadiuses;
float* brightnesses;
float* contrasts;
float* saturations;
float* hues;
float* shadowses;
float* highlightses;
float* temps;

char* shadowsLutYs;
char* highlightsLutYs;
float* mTes;
float* kTes;

static const float mTi = 1.f;
static const float kTi = 0.f;

// YCrCb
static const int RGB2YCC_C0 = 4899;
static const int RGB2YCC_C1 = 9617;
static const int RGB2YCC_C2 = 1868;
static const int RGB2YCC_C3 = 11682;
static const int RGB2YCC_C4 = 9241;

static const int YCC_SHIFT = 14;
static const int YCC_DELTA = 2097152; //128 * (1 << YCC_SHIFT);

static const int YCC2RGB_C0 = 22987;
static const int YCC2RGB_C1 = -11698;
static const int YCC2RGB_C2 = -5636;
static const int YCC2RGB_C3 = 29049;
static const float scale = 1.f / 255.f;

#define DESCALE(x, n) (((x) + (1 << ((n)-1))) >> (n))


void set_selective_adjustment_params(int w,int h,int labW,int labH,int _adjustPointsCount,float _maxTollerance){

	width = w;
	height = h;
	labWidth = labW;
	labHeight = labH;
	toLabScale = (float)labW/(float)width;
	labToBitmapScale = (float)width/(float)labW;
	adjustPointsCount = _adjustPointsCount;
	maxTollerance = _maxTollerance;
	
	//;
	/*
	rsDebug("testIntArray0 = ", rsGetElementAt_int(testIntArray,0));
	rsDebug("testIntArray1 = ", rsGetElementAt_int(testIntArray,1));
	rsDebug("testIntArray2 = ", rsGetElementAt_int(testIntArray,2));*/
	
}

static float fclamp(float v, float min_v, float max_v)
{
    return v <= min_v ? min_v : (v <= max_v ? v : max_v);
}

void calculate_highlight_shadows_temps()
{
	for (int i = 0; i < adjustPointsCount; i++)
	{

		float temp = temps[i];
		mTes[i] = temp > 0 ? -1.f : 1.f;
		kTes[i] = fabs(temp);
	}

	for(int i = 0; i < adjustPointsCount; ++i)
	{
		const float kh = highlightses[i] > 0 ? -1.f : 1.f;
		const float ks = shadowses[i] > 0 ? 1.f : -1.f;
		const float ah = 1 + 2.0f * fabs(highlightses[i]) / 100.0f;
		const float as = 1 + 2.0f * fabs(shadowses[i]) / 100.0f;
		const float b = 2.f;

		for (int j = 0; j < 256; ++j)
		{
			const float x = j / 255.f;
			const float omx = 1.f - x;
			const float ys = ks * (omx - pow(omx, as)) * pow(omx, b);
			const float yh = - kh * (x - pow(x, ah)) * pow(x, b);
			highlightsLutYs[i*256 + j] = (char) (yh * 255);
			shadowsLutYs[i*256 + j]    = (char) (ys * 255);
		}
	}
}

/*
void set_selective_adjustment_arrays(rs_allocation _centerXs){
	centerXs = _centerXs;
}*/

void printParameters(){
	
	rsDebug("adjustPointsCount = ", adjustPointsCount);
	
	rsDebug("width = ", width);
	rsDebug("height = ", height);
	rsDebug("labWidth = ", labWidth);
	rsDebug("labHeight = ", labHeight);
	rsDebug("toLabScale = ", toLabScale);
	rsDebug("labToBitmapScale = ", labToBitmapScale);
	
	for(int i = 0;i < adjustPointsCount;i++){
		rsDebug("centerX = ", centerXs[i]);
		rsDebug("centerY = ", centerYs[i]);
		rsDebug("radius = ", radiuses[i]);
		rsDebug("minRadius = ", minRadiuses[i]);
		rsDebug("brightness = ", brightnesses[i]);
		rsDebug("contrast = ", contrasts[i]);
		rsDebug("saturation = ", saturations[i]);
		rsDebug("hue = ", hues[i]);
		rsDebug("shadows = ", shadowses[i]);
		rsDebug("highlight = ", highlightses[i]);
		rsDebug("temp = ", temps[i]);
	}

}

/*
void set_selective_adjustment_arrays(int centerXs[]){
	
}*/

void set_selective_adjustment_diff0(rs_allocation diff0){
	diffs[0] = diff0;
}

void set_selective_adjustment_diff1(rs_allocation diff1){
	diffs[1] = diff1;
}

void set_selective_adjustment_diff2(rs_allocation diff2){
	diffs[2] = diff2;
}

void set_selective_adjustment_diff3(rs_allocation diff3){
	diffs[3] = diff3;
}

void set_selective_adjustment_diff4(rs_allocation diff4){
	diffs[4] = diff4;
}

void set_selective_adjustment_diff5(rs_allocation diff5){
	diffs[5] = diff5;
}

void set_selective_adjustment_diff6(rs_allocation diff6){
	diffs[6] = diff6;
}

void set_selective_adjustment_diff7(rs_allocation diff7){
	diffs[7] = diff7;
}

static uchar correctBrightness(uchar channel,float brightness){

	
	float value = channel/255.f;
	value = 1.f - value;
	value = 1.f - pow(value,brightness);
	channel = SATURATE_CAST_TO_UCHAR(value*255);
	return channel;
}

static void rabidRGBtoHSL(uchar red, uchar green, uchar blue, float HSL[3]) {

	float r = red / 255.f;
	float g = green / 255.f;
	float b = blue / 255.f;

	float H = 0.f;
	float S = 0.f;
	float L = 0.f;

	float cMax = fmax(fmax(r, g), b);
	float cMin = fmin(fmin(r, g), b);
	float delta = cMax - cMin;
	if (delta == 0.f) {
		H = 0.f;
	} else if (cMax == r) {
		H = fmod((g - b) / delta, 6.f) * 60.f;
	} else if (cMax == g) {
		H = ((b - r) / delta + 2.f) * 60.f;
	} else {
		H = ((r - g) / delta + 4.f) * 60.f;
	}

	if (H < 0.f) {
		H += 360.f;
	} else if (H > 360.f) {
		H -= 360.f;
	}

	L = (cMax + cMin) / 2.f;

	if (delta == 0.f) {
		S = 0.f;
	} else {
		S = delta / (1.f - fabs((2.f * L - 1.f)));
	}
	HSL[0] = H;
	HSL[1] = S;
	HSL[2] = L;

}


static void rabidHSLtoRGB(float hsl[3],uchar rgb[3]) {


	float H = hsl[0];
	float S = hsl[1];
	float L = hsl[2];

	float C = (1.0 - fabs((2.f * L - 1.f))) * S;
	float X = C * (1.f - fabs((fmod((H / 60.f), 2.f) - 1.f)));
	float m = L - C / 2.f;

	float r = 0.f;
	float g = 0.f;
	float b = 0.f;

	if (H >= 0.f && H <= 60.f) {
		r = C;
		g = X;
		b = 0.f;
	} else if (H >= 60.f && H <= 120.f) {
		r = X;
		g = C;
		b = 0.f;
	} else if (H >= 120.f && H <= 180.f) {
		r = 0.f;
		g = C;
		b = X;
	} else if (H >= 180.f && H <= 240.f) {
		r = 0.f;
		g = X;
		b = C;
	} else if (H >= 240.f && H <= 300.f) {
		r = X;
		g = 0.f;
		b = C;
	} else if (H >= 300.f && H <= 360.f) {
		r = C;
		g = 0.f;
		b = X;
	}

	r += m;
	g += m;
	b += m;
	rgb[0] = SATURATE_CAST_TO_UCHAR(r * 255.f);
	rgb[1] = SATURATE_CAST_TO_UCHAR(g * 255.f);
	rgb[2] = SATURATE_CAST_TO_UCHAR(b * 255.f);
}

void root(const uchar4 *v_in, uchar4 *v_out, uint32_t x, uint32_t y) {

	int labDataX = (int)(x*toLabScale);
	if(labDataX >= labWidth){
		labDataX = labWidth - 1;
	}
	
	int labDataY = (int)(y*toLabScale);
	
	if(labDataY >= labHeight){
		labDataY = labHeight - 1;
	}
	
	uchar destRed = v_in->r;
	uchar destGreen = v_in->g;
	uchar destBlue = v_in->b;

	const uchar origY = DESCALE(destRed * RGB2YCC_C0  +  destGreen * RGB2YCC_C1  +  destBlue * RGB2YCC_C2, YCC_SHIFT);
	const int Cr = DESCALE((destRed - origY) * RGB2YCC_C3 + YCC_DELTA, YCC_SHIFT);
	const int Cb = DESCALE((destBlue - origY) * RGB2YCC_C4 + YCC_DELTA, YCC_SHIFT);
	
	float maxStrength = 0;
	float minDistance = 10000000;
	
	float distances[adjustPointsCount];
	float strengths[adjustPointsCount];
	int activePoints[adjustPointsCount];
	float weights[adjustPointsCount];

	int activePointsCount = 0;
	
	for (int j = 0; j < adjustPointsCount; j++)
	{
		if(radiuses[j] == 0)
			continue;

		rs_allocation diffAllocation = diffs[j];
		float difference = rsGetElementAt_float(diffAllocation, labDataX, labDataY);
		activePoints[j] = 0;
		
		if (difference <= maxTollerance) {

				int centerX = centerXs[j];
				int centerY = centerYs[j];
				int dx = x - centerX;
				int dy = y - centerY;
				float dstance2 = dx*dx + dy*dy;
				float distance = sqrt(dstance2) + 1;
				activePointsCount++;

				{

					activePoints[j] = 1;
					float strength =
							((1.f - difference / maxTollerance) * 255.f);

					if (strength > 255) {
						strength = 255;
					}

					distance += 1;

					distances[j] = distance;
					strengths[j] = strength;

					if (distance < minDistance) {
						minDistance = distance;
					}

					if (strength > maxStrength) {
						maxStrength = strength;
					}

				}

			}
	}
	
	float weightSum = 0;

	for (int j = 0; j < adjustPointsCount; j++)
	{
		if(radiuses[j] == 0)
			continue;

		if (activePoints[j] == 1) {

			float weight = strengths[j]*minDistance/distances[j];
			weights[j] = weight*weight*weight;
			weightSum += weights[j];
		}

	}
	
	float deltaBrightness = 0;
	float deltaContrast = 0;
	float deltaSaturation = 0;
	float deltaHue = 0;
	float deltaShadow = 0;
	float deltaHighlight = 0;
	float deltaTemperatureR = 0.f;
	float deltaTemperatureG = 0.f;
	float deltaTemperatureB = 0.f;
	float deltaHues[8] =       {0,0,0,0,0,0,0,0};
	float hueMaskStreghts[8] = {0,0,0,0,0,0,0,0};

	float mainBrightness = 1.f;
	float mainContrast = 1.f;
	float radiusIntensity = 1.f;

	for (int j = 0; j < adjustPointsCount; j++)
	{
		if(radiuses[j] == 0)
			continue;

		float pointDistance = distances[j];
		int pointRadius = radiuses[j];
		
		if (activePoints[j] == 1
					&& pointDistance
							<= pointRadius ){
							
			float maskValue = maxStrength * weights[j] / weightSum;
			float colorValue = maskValue;

			radiusIntensity = (1- pointDistance / pointRadius)*(1-pointDistance / pointRadius);
			maskValue =  (maskValue*radiusIntensity );

			if (maskValue > 0) {

				if (maskValue > 255) {
					maskValue = 255;
				}
				
				float maskStrength = maskValue / 255.f;

				float pointBrightness = brightnesses[j];
				float pointContrast = contrasts[j];
				float pointSaturation = saturations[j];
				float pointHue = hues[j];
				float pointShadow = shadowsLutYs[j *256 + origY];
				float pointHilights = highlightsLutYs[j *256 + origY];
				float mTe = mTes[j];
				float kTe = kTes[j];

				float r = Cr * scale - 0.5f;
				float b = Cb * scale - 0.5f;
				float ti = b + r;
				float te = b - r;
				ti += (mTi - ti) * kTi;
				te += (mTe - te) * kTe;
				b = (ti + te) * 0.5f;
				r = ti - b;

				int tempCr = fclamp(r + 0.5f, 0.f, 1.f) * 255.f;
				int tempCb = fclamp(b + 0.5f, 0.f, 1.f) * 255.f;

				deltaTemperatureR += ( DESCALE((tempCr - 128) * YCC2RGB_C0, YCC_SHIFT) - DESCALE((Cr - 128) * YCC2RGB_C0, YCC_SHIFT) ) * maskStrength;
				deltaTemperatureG += ( DESCALE((tempCr - 128) * YCC2RGB_C1 + (tempCb - 128) * YCC2RGB_C2, YCC_SHIFT) - DESCALE((Cr - 128) * YCC2RGB_C1 + (Cb - 128) * YCC2RGB_C2, YCC_SHIFT) ) * maskStrength;
				deltaTemperatureB += ( DESCALE((tempCb - 128) * YCC2RGB_C3, YCC_SHIFT) -  DESCALE((Cb - 128) * YCC2RGB_C3, YCC_SHIFT)) * maskStrength;

				deltaBrightness += (pointBrightness - 1) * maskStrength;
				deltaContrast += (pointContrast - 1) * maskStrength;
				deltaSaturation += (pointSaturation - 1) * maskStrength;
				deltaHue = (pointHue)*colorValue/255;
				deltaShadow += (pointShadow - 1) * maskStrength;
				deltaHighlight += (pointHilights - 1) * maskStrength;

				deltaHues[j] = deltaHue;
				hueMaskStreghts[j] =  maskStrength;

				mainBrightness *= (1 + deltaBrightness*maskStrength);
				mainContrast *= (1 + deltaContrast*maskStrength);
				
			}
		}

	}

	for (int j = 0; j<adjustPointsCount; j++)
	{
		if(deltaHues[j] != 0){
			float hsl[3];
			rabidRGBtoHSL(destRed, destGreen, destBlue, hsl);
			hsl[0] += deltaHues[j];
			if(hsl[0] < 0.f){
				hsl[0] += 360.f;
			}else if(hsl[0] > 360.f){
				hsl[0] -= 360.f;
			}

			uchar rgb[3];
			rabidHSLtoRGB(hsl, rgb);

			destRed =  hueMaskStreghts[j]*rgb[0]+(1-hueMaskStreghts[j])*destRed;
			destGreen =  hueMaskStreghts[j]*rgb[1]+(1-hueMaskStreghts[j])*destGreen;
			destBlue =  hueMaskStreghts[j]*rgb[2]+(1-hueMaskStreghts[j])* destBlue;

		}
	}


	if (deltaBrightness != 0) {

		float brightness = 1 + deltaBrightness;
		destRed = correctBrightness(destRed,brightness);
		destGreen = correctBrightness(destGreen,brightness);
		destBlue = correctBrightness(destBlue,brightness);
	}
	
	
	if (deltaContrast != 0) {
		float contrast = 1 + deltaContrast;
		destRed = SATURATE_CAST_TO_UCHAR(
							((destRed - 127.5f) * contrast + 127.5f));
		destGreen = SATURATE_CAST_TO_UCHAR(
							((destGreen - 127.5f) * contrast + 127.5f));
		destBlue = SATURATE_CAST_TO_UCHAR(
							((destBlue - 127.5f) * contrast + 127.5f));
	}
	
	if (deltaSaturation != 0 || deltaHue != 0) {


		float hsl[3];
		rabidRGBtoHSL(destRed, destGreen, destBlue, hsl);
			float saturation = 1 + deltaSaturation;
			if (saturation < 1.f) {
				hsl[1] *= saturation;
			} else if (saturation > 1.f) {
				float sat = 1.0 - (saturation - 1.f);
				if (sat <= 0.f)
					sat = 0.0001f;
					hsl[1] /= sat;
			}

			hsl[1] = clamp(hsl[1],0.f,1.f);

			uchar rgb[3];
			rabidHSLtoRGB(hsl, rgb);
			destRed = rgb[0];
			destGreen = rgb[1];
			destBlue = rgb[2];

	}
	
	if( deltaTemperatureR !=0 && deltaTemperatureG !=0 && deltaTemperatureB !=0 )
	{
		destRed =   SATURATE_CAST_TO_UCHAR(destRed + deltaTemperatureR);
		destGreen = SATURATE_CAST_TO_UCHAR(destGreen + deltaTemperatureG);
		destBlue =  SATURATE_CAST_TO_UCHAR(destBlue + deltaTemperatureB);
	}

	if(deltaShadow != 0)
	{
		destRed =   SATURATE_CAST_TO_UCHAR(destRed + deltaShadow +1);
		destGreen = SATURATE_CAST_TO_UCHAR(destGreen + deltaShadow +1);
		destBlue =  SATURATE_CAST_TO_UCHAR(destBlue + deltaShadow +1);
	}

	if(deltaHighlight != 0)
	{
		destRed =   SATURATE_CAST_TO_UCHAR(destRed + deltaHighlight +1);
		destGreen = SATURATE_CAST_TO_UCHAR(destGreen + deltaHighlight +1);
		destBlue =  SATURATE_CAST_TO_UCHAR(destBlue + deltaHighlight +1);
	}

	
	v_out->r = destRed;
    v_out->g = destGreen;
    v_out->b = destBlue;
    v_out->a = v_in->a;

}
