#pragma version(1)
#pragma rs java_package_name(com.picsart.effects.rs)

#define SATURATE_CAST_TO_UCHAR(v) ((uchar)((v) <= 0 ? 0 : ((v) >= 255 ? 255 : (v))))

static rs_allocation diffs[8];

static int width;
static int height;

static int labWidth;
static int labHeight;
static float toLabScale;
static float labToBitmapScale;
static int adjustPointsCount;
static float maxTollerance;
static int selectedPointIndex;

int* centerXs;
int* centerYs;
int* radiuses;
int* minRadiuses;
float* brightnesses;


void set_selective_adjustment_mask_params(int w,int h,int labW,int labH,int _adjustPointsCount,int _selectedPointIndex,float _maxTollerance){

	width = w;
	height = h;
	labWidth = labW;
	labHeight = labH;
	toLabScale = (float)labW/(float)width;
	labToBitmapScale = (float)width/(float)labW;
	adjustPointsCount = _adjustPointsCount;
	selectedPointIndex = _selectedPointIndex;
	maxTollerance = _maxTollerance;
	
	
	
}

void set_selective_adjustment_mask_diff0(rs_allocation diff0){
	diffs[0] = diff0;
}

void set_selective_adjustment_mask_diff1(rs_allocation diff1){
	diffs[1] = diff1;
}

void set_selective_adjustment_mask_diff2(rs_allocation diff2){
	diffs[2] = diff2;
}

void set_selective_adjustment_mask_diff3(rs_allocation diff3){
	diffs[3] = diff3;
}

void set_selective_adjustment_mask_diff4(rs_allocation diff4){
	diffs[4] = diff4;
}

void set_selective_adjustment_mask_diff5(rs_allocation diff5){
	diffs[5] = diff5;
}

void set_selective_adjustment_mask_diff6(rs_allocation diff6){
	diffs[6] = diff6;
}

void set_selective_adjustment_mask_diff7(rs_allocation diff7){
	diffs[7] = diff7;
}

void root(const uchar *v_in, uchar *v_out, uint32_t x, uint32_t y) {

	int labDataX = (int)(x*toLabScale);
	if(labDataX >= labWidth){
		labDataX = labWidth - 1;
	}
	
	int labDataY = (int)(y*toLabScale);
	
	if(labDataY >= labHeight){
		labDataY = labHeight - 1;
	}
	
	
	float maxStrength = 0;
	float minDistance = 10000000;
	
	float distances[adjustPointsCount];
	float strengths[adjustPointsCount];
	int activePoints[adjustPointsCount];
	float weights[adjustPointsCount];

	int activePointsCount = 0;
	
	for (int j = 0; j < adjustPointsCount; j++) {
		if (radiuses[j] == 0 )
			continue;

		rs_allocation diffAllocation = diffs[j];
		float difference = rsGetElementAt_float(diffAllocation, labDataX, labDataY);
		activePoints[j] = 0;
		
		if (difference <= maxTollerance) {

				int centerX = centerXs[j];
				int centerY = centerYs[j];
				int dx = x - centerX;
				int dy = y - centerY;
				float dstance2 = dx*dx + dy*dy;
				float distance = sqrt(dstance2) + 1;
				activePointsCount++;

				{

					activePoints[j] = 1;
					float strength =
							((1.f - difference / maxTollerance) * 255.f);

					if (strength > 255) {
						strength = 255;
					}

					distance += 1;

					distances[j] = distance;
					strengths[j] = strength;

					if (distance < minDistance) {
						minDistance = distance;
					}

					if (strength > maxStrength) {
						maxStrength = strength;
					}

				}

			}
	}
	
	float weightSum = 0;
	float maskValue = 0;

	for (int j = 0; j < adjustPointsCount; j++)
	{
		if( radiuses[j] == 0)
			continue;

		if (activePoints[j] == 1) {

			float weight = strengths[j]*minDistance/distances[j];
			weights[j] = weight*weight*weight;
			weightSum += weights[j];
		}
	}

	//float radiusIntensity = 1.f;
	if( distances[selectedPointIndex] == 0)
		return;

	float pointDistance = distances[selectedPointIndex];
		
	int pointRadius = radiuses[selectedPointIndex];
	if (activePoints[selectedPointIndex] == 1
		&& pointDistance <= pointRadius )
	{
		maskValue =  (maxStrength * weights[selectedPointIndex] / weightSum);
		float radiusIntensity = (1- pointDistance / pointRadius)*(1-pointDistance / pointRadius);
		maskValue =  (maskValue*radiusIntensity );
		if (maskValue > 0)
			maskValue = SATURATE_CAST_TO_UCHAR(maskValue);
	}

	
	*v_out = maskValue;

}
