
#pragma version(1)
#pragma rs java_package_name(com.picsart.effects.rs)

#include "rs_utils.rsh"

static short lutY[256];

static void generate_lutY(const float s_k, const float s_a, const float h_k, const float h_a) {
    for (int i = 0; i < 256; ++i) {
        const float x = i / 255.f;
        const float omx = 1.f - x;
        const float y = s_k * (omx - pow(omx, s_a)) * omx * omx - h_k * (x - pow(x, h_a)) * x * x;
        lutY[i] = y * 255;
    }
}

void set_params_shadows_and_highlights(int amount, bool highlights) {
    const float k = highlights ? (amount > 0 ? -1.f : 1.f) : (amount > 0 ? 1.f : -1.f);
    const float a = 1 + 2.0f * abs(amount) / 100.0f;
    if (highlights)
        generate_lutY(0.f, 0.f, k, a);
    else
        generate_lutY(k, a, 0.f, 0.f);
}

void root(const uchar4 *v_in, uchar4 *v_out, uint32_t x, uint32_t y) {
    int r = v_in->r;
    int g = v_in->g;
    int b = v_in->b;

    const int origY = DESCALE(r * RGB2YCC_C0  +  g * RGB2YCC_C1  +  b * RGB2YCC_C2, YCC_SHIFT);
    const short delta = lutY[origY];

    r = SATURATE_CAST_TO_UCHAR(r + delta);
    g = SATURATE_CAST_TO_UCHAR(g + delta);
    b = SATURATE_CAST_TO_UCHAR(b + delta);
    *v_out = (uchar4){r, g, b, 255};
}
