
#pragma version(1)
#pragma rs java_package_name(com.picsart.effects.rs)

#include "rs_utils.rsh"

static int centerX, centerY;
static float radius;

static const uchar sharpen_dodger_RGB[] = {0, 1, 2, 2, 3, 4, 5, 5, 6, 7, 8, 8, 9, 10, 11, 11, 11, 12, 13, 13, 14, 15, 16, 16, 17, 18, 19, 19, 20, 21, 22, 23, 23, 24, 25, 26, 26, 27, 28, 29, 29, 30, 31, 32, 32, 33, 34, 35, 35, 36, 37, 38, 38, 39, 40, 41, 41, 42, 43, 45, 46, 47, 49, 50, 51, 52, 53, 54, 55, 56, 58, 59, 60, 61, 62, 63, 65, 67, 68, 69, 70, 71, 72, 73, 75, 76, 78, 79, 80, 81, 82, 83, 85, 86, 88, 89, 90, 91, 92, 94, 95, 97, 98, 99, 100, 101, 102, 105, 106, 107, 108, 109, 110, 111, 114, 115, 116, 117, 118, 120, 121, 123, 124, 125, 127, 128, 129, 130, 131, 134, 135, 136, 137, 139, 140, 141, 143, 144, 146, 147, 148, 149, 150, 151, 154, 155, 156, 157, 158, 160, 161, 163, 164, 165, 166, 168, 169, 170, 172, 173, 174, 175, 177, 178, 179, 180, 182, 183, 184, 185, 186, 187, 189, 191, 192, 193, 194, 195, 196, 197, 198, 200, 201, 202, 203, 204, 205, 206, 208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222, 223, 224, 225, 226, 227, 227, 228, 229, 229, 230, 231, 232, 232, 233, 234, 235, 235, 236, 236, 237, 238, 238, 239, 240, 240, 241, 241, 242, 243, 243, 244, 245, 245, 245, 246, 247, 247, 248, 248, 249, 249, 250, 250, 251, 251, 252, 252, 253, 253, 253, 254, 254, 255};

static void sharpen_dodger_gradient_map(uchar4 *dp) {
    int r = dp->r;
    int g = dp->g;
    int b = dp->b;

    const int r0 = 10;
    const int g0 = 47;
    const int b0 = 89;
    const int dr = 222;
    const int dg = 123;
    const int db = 23;

    int Y = PST_GRADIENT_SMOOTH_100_TABLE[ DESCALE(r * RGB2YCC_C0  +  g * RGB2YCC_C1  +  b * RGB2YCC_C2, YCC_SHIFT) ];

    int tr = max(r0 + dr * Y / 255, r);
    int tg = max(g0 + dg * Y / 255, g);
    int tb = max(b0 + db * Y / 255, b);

    dp->r = (tr * 7 + r * 3) / 10;
    dp->g = (tg * 7 + g * 3) / 10;
    dp->b = (tb * 7 + b * 3) / 10;
}

static void sharpen_dodger_radial_gradient(uchar4 *dp, uint32_t x, uint32_t y) {
    const int r0 = 243;
    const int g0 = 243;
    const int b0 = 243;
    const int dr = -205;
    const int dg = -197;
    const int db = -186;

    const int dx = x - centerX;
    const int dy = y - centerY;
    const int k = PST_GRADIENT_SMOOTH_100_TABLE[ min((int)(sqrt((float)(dx*dx + dy*dy)) / radius * 255.f + 0.5f), 255) ];

    const int gr = 2 * (r0 + dr * k / 255) - 255;
    const int gg = 2 * (g0 + dg * k / 255) - 255;
    const int gb = 2 * (b0 + db * k / 255) - 255;
    const int omk = 255 - k;

    const int r = dp->r;
    const int g = dp->g;
    const int b = dp->b;

    const int tr = r + gr * r * (255 - r) / 65025;
    const int tg = g + gg * g * (255 - g) / 65025;
    const int tb = b + gb * b * (255 - b) / 65025;

    dp->r = (tr * k + r * omk) / 255;
    dp->g = (tg * k + g * omk) / 255;
    dp->b = (tb * k + b * omk) / 255;
}

void set_params_sharpen_dodger(int w, int h) {
    centerX = (w - 1) / 2;
    centerY = (h - 1) / 2;
    radius = w * 0.75f;
}

void root(const uchar4 *v_in, uchar4 *v_out, uint32_t x, uint32_t y) {
    uchar4 val = *v_in;

    // Gradient Map
    {
        sharpen_dodger_gradient_map(&val);
    }

    // Curves, Brightness & Contrast
    {
        val.r = sharpen_dodger_RGB[val.r];
        val.g = sharpen_dodger_RGB[val.g];
        val.b = sharpen_dodger_RGB[val.b];
    }

    // Radial Gradient
    {
        sharpen_dodger_radial_gradient(&val, x, y);
    }

    *v_out = val;
}
