
#pragma version(1)
#pragma rs java_package_name(com.picsart.effects.rs)

#include "rs_utils.rsh"

static rs_allocation gray_data, overlay_gray;
static int W, H, wm1, hm1, l, c, b;

void set_params_sketchup(rs_allocation src_gray, rs_allocation gray_overlay, int lines, int brightness, int contrast) {
    W = rsAllocationGetDimX(src_gray);
    H = rsAllocationGetDimY(src_gray);
    wm1 = W - 1;
    hm1 = H - 1;

    gray_data = src_gray;
    overlay_gray = gray_overlay;

    l = min(100, max(lines, 0));
    b = min(100, max(brightness, 0));
    c = min(100, max(contrast, 0));
}

void root(uchar4 *v_out, uint32_t x, uint32_t y) {
	int ym1 = y - 1;
    int yp1 = y + 1;
    int xm1 = x - 1;
    int xp1 = x + 1;
    if (x == 0 || x == wm1 || y == 0 || y == hm1) {
        if( ym1 < 0) ym1 = 0;
		else if(yp1 > hm1) yp1 = hm1;
		if( xm1 < 0) xm1 = 0;
        else if(xp1 > wm1) xp1 = wm1;
    }

    const uchar* psgr = getRowUChar(gray_data, ym1); // previous source gray row
	const uchar* csgr = getRowUChar(gray_data, y    ); // current source gray row
	const uchar* nsgr = getRowUChar(gray_data, yp1); // next source gray row

    // Detect lines using Sobel edge detector.
    int h_grad = (psgr[xm1] + 2 * psgr[x] + psgr[xp1]) - (nsgr[xm1] + 2 * nsgr[x] + nsgr[xp1]);
    int v_grad = (psgr[xm1] + 2 * csgr[xm1] + nsgr[xm1]) - (psgr[xp1] + 2 * csgr[xp1] + nsgr[xp1]);
    int result = SATURATE_CAST_TO_UCHAR((abs(h_grad) + abs(v_grad)) * l / 50);

    // Adjust edges.
    result = 255 - result * (382 - result) / 255;

    // Blend edges with with overlay.
    int Y = csgr[x];
    if (Y < 26)
        result = result * getValUChar(overlay_gray, (y + H) / 2, (x + W) / 2) / 255;
    else if (Y < 77)
        result = result * getValUChar(overlay_gray, (y + H) / 2, (x    ) / 2) / 255;
    else if (Y < 128)
        result = result * getValUChar(overlay_gray, (y    ) / 2, (x + W) / 2) / 255;
    else if (Y < 179)
        result = result * getValUChar(overlay_gray, (y    ) / 2, (x    ) / 2) / 255;

    // Contrast and brightness.
    result = SATURATE_CAST_TO_UCHAR(((result - 128) * c / 50 + 128) * b / 50);

    // Store results in destination image.
    v_out->r = result;
    v_out->g = result;
    v_out->b = result;
    v_out->a = 255;
}
