
#pragma version(1)
#pragma rs java_package_name(com.picsart.effects.rs)

#include "rs_utils.rsh"

static rs_allocation gray_data, blured_data;
static int W, H, inv_amount;

void set_params_smartblur(rs_allocation src_gray, rs_allocation blured, int amount) {
    W = rsAllocationGetDimX(src_gray);
    H = rsAllocationGetDimY(src_gray);

    gray_data = src_gray;
    blured_data = blured;

    inv_amount = 100 - min(100, max(0, amount));
}

void root(const uchar4 *v_in, uchar4 *v_out, uint32_t x, uint32_t y) {
    if (x == 0 || x == (W - 1) || y == 0 || y == (H - 1)) {
        *v_out = (uchar4){0, 0, 0, 255};
        return;
    }

    const uchar* psgr = getRowUChar(gray_data, y - 1); // previous source gray row
    const uchar* csgr = getRowUChar(gray_data, y    ); // current source gray row
    const uchar* nsgr = getRowUChar(gray_data, y + 1); // next source gray row
    const uchar4* cbcp = getUChar4(blured_data, y, x); // current blured color

    // Detect lines using Sobel edge detector.
    const int h_grad = (psgr[x - 1] + 2 * psgr[x] + psgr[x + 1]) - (nsgr[x - 1] + 2 * nsgr[x] + nsgr[x + 1]);
    const int v_grad = (psgr[x - 1] + 2 * csgr[x - 1] + nsgr[x - 1]) - (psgr[x + 1] + 2 * csgr[x + 1] + nsgr[x + 1]);
    const uchar variability = SATURATE_CAST_TO_UCHAR((abs(h_grad) + abs(v_grad)) * inv_amount / 20);
    const uchar inv_variability = 255 - variability;

    // Store results in destination image.
    v_out->r = (v_in->r * variability + cbcp->r * inv_variability) / 255;
    v_out->g = (v_in->g * variability + cbcp->g * inv_variability) / 255;
    v_out->b = (v_in->b * variability + cbcp->b * inv_variability) / 255;
    v_out->a = 255;
}
