
#pragma version(1)
#pragma rs java_package_name(com.picsart.effects.rs)

#include "rs_utils.rsh"

enum BlendMode {
   BlendModeNormal,
   BlendModeMultiply,
   BlendModeScreen,
   BlendModeOverlay,
   BlendModeDarken,
   BlendModeLighten,
   BlendModeColorDodge,
   BlendModeColorBurn,
   BlendModeSoftLight,
   BlendModeHardLight,
   BlendModeDifference,
   BlendModeExclusion,
   BlendModeHue,
   BlendModeSaturation,
   BlendModeColor,
   BlendModeLuminosity,
   BlendModeClear,
   BlendModeCopy,
   BlendModeSourceIn,
   BlendModeSourceOut,
   BlendModeSourceAtop,
   BlendModeDestinationOver,
   BlendModeDestinationIn,
   BlendModeDestinationOut,
   BlendModeDestinationAtop,
   BlendModeXOR,
   BlendModePlusDarker,
   BlendModePlusLighter
};

static const unsigned char blend_func_multiply(const unsigned char Sc, const unsigned char Dc) {
    return (Sc * Dc) / 255;
}

static const unsigned char blend_func_screen(const unsigned char Sc, const unsigned char Dc) {
    return Sc + Dc - Sc * Dc / 255;
}

static const unsigned char blend_func_overlay(const unsigned char Sc, const unsigned char Dc) {
    if (Dc < 128)
        return 2 * Sc * Dc / 255;
    else
        return 255 - 2 * (255 - Dc) * (255 - Sc) / 255;
}

static const unsigned char blend_func_darken(const unsigned char Sc, const unsigned char Dc) {
    return min(Sc,Dc);
}

static const unsigned char blend_func_lighten(const unsigned char Sc, const unsigned char Dc) {
    return max(Sc,Dc);
}

static const unsigned char blend_func_color_dodge(const unsigned char Sc, const unsigned char Dc) {
    if (Sc == 255)
        return 255;
    else
        return min(255,  255 * Dc / (255 - Sc));
}

static const unsigned char blend_func_color_burn(const unsigned char Sc, const unsigned char Dc) {
    if (Sc == 0)
        return 0;
    else
        return 255 - min(255, 255 * (255 - Dc) / Sc);
}

static const unsigned char blend_func_soft_light(const unsigned char Sc, const unsigned char Dc) {
    return Dc - (255 - 2 * Sc) * Dc * (255 - Dc) / 65025;
}

static const unsigned char blend_func_hard_light(const unsigned char Sc, const unsigned char Dc) {
    if (Sc < 128)
        return 2 * Sc * Dc / 255;
    else
        return 255 - 2 * (255 - Dc) * (255 - Sc) / 255;
}

static const unsigned char blend_func_difference(const unsigned char Sc, const unsigned char Dc) {
    return abs((int)Dc - Sc);
}

static const unsigned char blend_func_exclusion(const unsigned char Sc, const unsigned char Dc) {
    return Sc + Dc - 2 * Sc * Dc / 255;
}

static const unsigned char blend_func_plus_darker(const unsigned char Sc, const unsigned char Dc) {
    return max(0, Sc + Dc - 255);
}

static const unsigned char blend_func_plus_lighter(const unsigned char Sc, const unsigned char Dc) {
    return min(255, Sc + Dc);
}

static rs_allocation top, bottom;
//static const unsigned char (*blend_func) (const unsigned char Sc, const unsigned char Dc);
static int _blend_mode;

void set_params_utils_blend_with_mode(rs_allocation top_a, rs_allocation bottom_a, int blend_mode) {
    top = top_a;
    bottom = bottom_a;
	/*
    switch (blend_mode) {
        case BlendModeMultiply:
            blend_func = blend_func_multiply;
            break;
        case BlendModeScreen:
            blend_func = blend_func_screen;
            break;
        case BlendModeOverlay:
            blend_func = blend_func_overlay;
            break;
        case BlendModeDarken:
            blend_func = blend_func_darken;
            break;
        case BlendModeLighten:
            blend_func = blend_func_lighten;
            break;
        case BlendModeColorDodge:
            blend_func = blend_func_color_dodge;
            break;
        case BlendModeColorBurn:
            blend_func = blend_func_color_burn;
            break;
        case BlendModeSoftLight:
            blend_func = blend_func_soft_light;
            break;
        case BlendModeHardLight:
            blend_func = blend_func_hard_light;
            break;
        case BlendModeDifference:
            blend_func = blend_func_difference;
            break;
        case BlendModeExclusion:
            blend_func = blend_func_exclusion;
            break;
        case BlendModePlusDarker:
            blend_func = blend_func_plus_darker;
            break;
        case BlendModePlusLighter:
            blend_func = blend_func_plus_lighter;
            break;
        default:
            return;
        }
    */
    _blend_mode = blend_mode;
}

void root(uchar4 *v_out, uint32_t x, uint32_t y) {
    const uchar4 t = getValUChar4(top, y, x);
    const uchar4 b = getValUChar4(bottom, y, x);
    
    /*
    *v_out = (uchar4) { blend_func(t.r, b.r),
                        blend_func(t.g, b.g),
                        blend_func(t.b, b.b),
                        255 };
    */

    switch(_blend_mode)
    {
        case BlendModeMultiply:
            *v_out = (uchar4) { blend_func_multiply(t.r, b.r),
                            blend_func_multiply(t.g, b.g),
                            blend_func_multiply(t.b, b.b),
                            255 };
        break;

        case BlendModeScreen:
            *v_out = (uchar4) { blend_func_screen(t.r, b.r),
                            blend_func_screen(t.g, b.g),
                            blend_func_screen(t.b, b.b),
                            255 };
        break;

        case BlendModeOverlay:
            *v_out = (uchar4) { blend_func_overlay(t.r, b.r),
                            blend_func_overlay(t.g, b.g),
                            blend_func_overlay(t.b, b.b),
                            255 };
        break;

        case BlendModeDarken:
            *v_out = (uchar4) { blend_func_darken(t.r, b.r),
                            blend_func_darken(t.g, b.g),
                            blend_func_darken(t.b, b.b),
                            255 };
        break;

        case BlendModeLighten:
            *v_out = (uchar4) { blend_func_lighten(t.r, b.r),
                            blend_func_lighten(t.g, b.g),
                            blend_func_lighten(t.b, b.b),
                            255 };
        break;

        case BlendModeColorDodge:
            *v_out = (uchar4) { blend_func_color_dodge(t.r, b.r),
                            blend_func_color_dodge(t.g, b.g),
                            blend_func_color_dodge(t.b, b.b),
                            255 };
        break;

        case BlendModeColorBurn:
            *v_out = (uchar4) { blend_func_color_burn(t.r, b.r),
                            blend_func_color_burn(t.g, b.g),
                            blend_func_color_burn(t.b, b.b),
                            255 };
        break;

        case BlendModeSoftLight:
            *v_out = (uchar4) { blend_func_soft_light(t.r, b.r),
                            blend_func_soft_light(t.g, b.g),
                            blend_func_soft_light(t.b, b.b),
                            255 };
        break;

        case BlendModeHardLight:
            *v_out = (uchar4) { blend_func_hard_light(t.r, b.r),
                            blend_func_hard_light(t.g, b.g),
                            blend_func_hard_light(t.b, b.b),
                            255 };
        break;

        case BlendModeDifference:
            *v_out = (uchar4) { blend_func_difference(t.r, b.r),
                            blend_func_difference(t.g, b.g),
                            blend_func_difference(t.b, b.b),
                            255 };
        break;

        case BlendModeExclusion:
            *v_out = (uchar4) { blend_func_exclusion(t.r, b.r),
                            blend_func_exclusion(t.g, b.g),
                            blend_func_exclusion(t.b, b.b),
                            255 };
        break;

        case BlendModePlusDarker:
            *v_out = (uchar4) { blend_func_plus_darker(t.r, b.r),
                            blend_func_plus_darker(t.g, b.g),
                            blend_func_plus_darker(t.b, b.b),
                            255 };
        break;

        case BlendModePlusLighter:
            *v_out = (uchar4) { blend_func_plus_lighter(t.r, b.r),
                            blend_func_plus_lighter(t.g, b.g),
                            blend_func_plus_lighter(t.b, b.b),
                            255 };
        break;
    }
}
