#pragma version(1)
#pragma rs java_package_name(com.picsart.effects.rs)

#include "rs_utils.rsh"

static rs_allocation integral;
static int W, H;
static int iblur;

void set_params_utils_box_convolve(rs_allocation source, rs_allocation temp, int blur) {
    W = rsAllocationGetDimX(source);
    H = rsAllocationGetDimY(source);

    integral = temp;

    iblur = blur / 2;

    const uchar4 *sp = getUChar4(source, 0, 0); // Source point
    int4 *cip = getInt4(integral, 0, 0);        // Current integral point
    cip->r = sp->r;
    cip->g = sp->g;
    cip->b = sp->b;

    sp = getUChar4(source, 0, 1); // Source point
    int4 *lip = getInt4(integral, 0, 0); // Left integral point
    cip       = getInt4(integral, 0, 1); // Current integral point
    for(int x = 1; x < W; ++x, sp++, lip++, cip++) {
        cip->r = sp->r + lip->r;
        cip->g = sp->g + lip->g;
        cip->b = sp->b + lip->b;
    }

    sp = getUChar4(source, 1, 0);
    int4 *tip = getInt4(integral, 0, 0); // Top integral point
    cip      = getInt4(integral, 1, 0);
    for(int y = 1; y < H; ++y, sp += W, tip += W, cip += W) {
        cip->r = sp->r + tip->r;
        cip->g = sp->g + tip->g;
        cip->b = sp->b + tip->b;
    }

    int4 *dip; // Diagonal (top-left) integral point
    for(int y = 1; y < H; ++y) {
        sp = getUChar4(source, y, 1);
        tip = getInt4(integral, y - 1, 1);
        dip = getInt4(integral, y - 1, 0);
        lip = getInt4(integral, y,     0);
        cip = getInt4(integral, y,     1);
        for(int x = 1; x < W; ++x, sp++, tip++, dip++, lip++, cip++) {
            cip->r = sp->r + lip->r + tip->r - dip->r;
            cip->g = sp->g + lip->g + tip->g - dip->g;
            cip->b = sp->b + lip->b + tip->b - dip->b;
        }
    }
}

void root(uchar4 *v_out, uint32_t x, uint32_t y) {
	const int x0 = max(0, (int)x - iblur);
	const int x1 = min(W - 1, (int)x + iblur);
	const int y0 = max(0, (int)y - iblur);
	const int y1 = min(H - 1, (int)y + iblur);

    const int S = (x1 - x0 + 1) * (y1 - y0 + 1);

    const int4 int_a = getValInt4(integral, y1, x1);
    const int4 int_b = x0 == 0 ? 0 : getValInt4(integral, y1, x0 - 1);
    const int4 int_c = y0 == 0 ? 0 : getValInt4(integral, y0 - 1, x1);
    const int4 int_d = (x0 == 0 || y0 == 0) ? 0 : getValInt4(integral, y0 - 1, x0 - 1);

    const int4 res = (int_a + int_d + S / 2 - int_b - int_c) / S;
    *v_out = (uchar4) {res.r, res.g, res.b, 255};
}
