
#pragma version(1)
#pragma rs java_package_name(com.picsart.effects.rs)

#include "rs_utils.rsh"

static int pst_get_luminance(const int r, const int g, const int b) {
    return DESCALE(r * RGB2YCC_C0  +  g * RGB2YCC_C1  +  b * RGB2YCC_C2, YCC_SHIFT);
}

static void pst_clip_colour_int(int * const r, int * const g, int * const b) {
    const int _r = *r;
    const int _g = *g;
    const int _b = *b;

    const int luminance = pst_get_luminance(_r, _g, _b);

    const int cMin = min(_r, min(_g, _b));
    const int cMax = max(_r, max(_g, _b));

    if (cMin < 0) {
        const int d = luminance - cMin;

        *r = luminance * (_r - cMin) / d;
        *g = luminance * (_g - cMin) / d;
        *b = luminance * (_b - cMin) / d;
    }

    if (cMax > 255) {
        const int d = cMax - luminance;
        const int k = 255 - luminance;
        const int c = luminance * (cMax - 255);

        *r = (_r * k + c) / d;
        *g = (_g * k + c) / d;
        *b = (_b * k + c) / d;
    }
}

static void pst_set_luminance(int * const r, int * const g, int * const b, const int luminance) {
    const int diff = luminance - pst_get_luminance(*r, *g, *b);
    if (diff == 0)
        return;

    *r += diff;
    *g += diff;
    *b += diff;

    pst_clip_colour_int(r, g, b);
}

static int pst_get_saturation(const int r, const int g, const int b) {
    const int cMin = min(r, min(g, b));
    const int cMax = max(r, max(g, b));
    return cMax - cMin;
}

static void pst_set_saturation(int * const r, int * const g, int * const b, const int saturation) {
    int * cMin;
    int * cMid;
    int * cMax;

    if (*r > *g) {
        if (*g > *b) {
            cMin = b;
            cMid = g;
            cMax = r;
        }
        else if (*r > *b) {
            cMin = g;
            cMid = b;
            cMax = r;
        }
        else {
            cMin = g;
            cMid = r;
            cMax = b;
        }
    } else {
        if (*r > *b) {
            cMin = b;
            cMid = r;
            cMax = g;
        }
        else if (*g > *b) {
            cMin = r;
            cMid = b;
            cMax = g;
        }
        else {
            cMin = r;
            cMid = g;
            cMax = b;
        }
    }

    if (*cMax > *cMin) {
        *cMid = ((*cMid - *cMin) * saturation) / (*cMax - *cMin);
        *cMax = saturation;
    }
    else {
        *cMid = 0;
        *cMax = 0;
    }
    *cMin = 0;
}
