
#pragma version(1)
#pragma rs java_package_name(com.picsart.effects.rs)

#include "rs_utils.rsh"

void root(uchar4 *v_out, uint32_t x, uint32_t y) {
    int r = v_out->r;
    int g = v_out->g;
    int b = v_out->b;

    // Selective color: Reds: cyan -28%, black 31%; Neutrals: black -12%
    {
        const int coefsR[] = {-15,  79,  79};
        const int coefsN[] = {-31, -31, -31};

        int metricR = 0;
        int metricN = 0;

        int minC, maxC;

        if (r > g) {
            if (r > b) {
                if (g > b) { // R > G > B
                    maxC = r;
                    minC = b;
                    metricR = r - g;
                }
                else { // R > B > G
                    maxC = r;
                    minC = g;
                    metricR = r - b;
                }
            }
            else { // B > R > G
                maxC = b;
                minC = g;
            }
        }
        else if (g > b) {
            if (r > b) { // G > R > B
                maxC = g;
                minC = b;
            }
            else { // G > B > R
                maxC = g;
                minC = r;
            }
        }
        else { // B > G > R
            maxC = b;
            minC = r;
        }

        const int bgw = 255 - (maxC - minC);
        if (bgw > 0) {
            if (minC >= 128) {
                metricN = bgw * (255 - minC) / 127;
            }
            else {
                metricN = bgw * minC / 127;
            }
        }

        int tr = 0;
        int tg = 0;
        int tb = 0;

        if (metricR > 0) {
            tr += metricR * SATURATE_CAST_TO_UCHAR(r - coefsR[0]);
            tg += metricR * SATURATE_CAST_TO_UCHAR(g - coefsR[1]);
            tb += metricR * SATURATE_CAST_TO_UCHAR(b - coefsR[2]);
        }

        if (metricN > 0) {
            tr += metricN * SATURATE_CAST_TO_UCHAR(r - coefsN[0]);
            tg += metricN * SATURATE_CAST_TO_UCHAR(g - coefsN[1]);
            tb += metricN * SATURATE_CAST_TO_UCHAR(b - coefsN[2]);
        }

        int other = 255 - metricR - metricN;

        r = (tr + other * r) / 255;
        g = (tg + other * g) / 255;
        b = (tb + other * b) / 255;
    }

    // Hue 13, Saturation 15
    {
        // Convert RGB to HSL
        int h = 0, s = 0, l;
        int vmin, vmax, diff;

        vmax = vmin = r;
        if( vmax < g ) vmax = g;
        if( vmax < b ) vmax = b;
        if( vmin > g ) vmin = g;
        if( vmin > b ) vmin = b;

        diff = vmax - vmin;
        l = (vmax + vmin) / 2;

        if( diff > 0 ) {
            s = (l < 128) ? (255 * diff / (vmax + vmin)) : (255 * diff / (510 - vmax - vmin));

            if( vmax == r )
                h = (g - b) * 60 / diff;
            else if( vmax == g )
                h = (b - r) * 60 / diff + 120;
            else
                h = (r - g) * 60 / diff + 240;

            if (h < 0)
                h += 360;
        }

        // Apply correction
        h = (h + 13) % 360;
        s = min(s * 100 / 85, 255);

        // Convert HSL to RGB
        static const int sector_data[][3] = {{0,3,1}, {2,0,1}, {1,0,3}, {1,2,0}, {3,1,0}, {0,1,2}};

        int p2 = (l < 128) ? (l * (255 + s) / 255) : (l + s - l * s / 255);
        int p1 = 2 * l - p2;

        uchar sector = h / 60;
        h = h % 60;

        uchar tab[4];
        tab[0] = p2;
        tab[1] = p1;
        tab[2] = p1 + (p2 - p1) * (60 - h) / 60;
        tab[3] = p1 + (p2 - p1) * h / 60;

        r = tab[sector_data[sector][0]];
        g = tab[sector_data[sector][1]];
        b = tab[sector_data[sector][2]];
    }

    *v_out = (uchar4){r, g, b, 255};
}
